/*
    SPDX-FileCopyrightText: 2017-2026 Laurent Montel <montel@kde.org>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#pragma once

#include <QWidget>

class AddTransportDialogNG_gui : public QWidget
{
    Q_OBJECT
public:
    explicit AddTransportDialogNG_gui(QWidget *parent = nullptr);
};
